@@ -2,6 +2,9 @@ import datetime
 import os
 import tempfile
 import typing
+import logging
+import json
+
 from dataclasses import dataclass
 
 from dominus.dominus_pb2 import (
@@ -23,6 +26,54 @@ class IngestionSessionContext:
     assets_output_location: str
     errors_output_location: str
     relationships_output_location: str
+    debug_metadata_path: str = None
+    debug_asset_to_attributes: typing.Dict[str, typing.Set[str]] = None
+
+    def __post_init__(self):
+        if self.debug_metadata_path is not None:
+            if os.path.exists(self.debug_metadata_path):
+                with open(self.debug_metadata_path, "r") as f:
+                    local_metadata = json.load(f)
+                    if "resources" in local_metadata:
+                        self.debug_asset_to_attributes = {}
+                        for r in local_metadata["resources"]:
+                            if "schema" in r and "attributes" in r["schema"]:
+                                for attr in r["schema"]["attributes"]:
+                                    if (
+                                        r["code"]
+                                        not in self.debug_asset_to_attributes
+                                    ):
+                                        self.debug_asset_to_attributes[
+                                            r["code"]
+                                        ] = set()
+                                    self.debug_asset_to_attributes[
+                                        r["code"]
+                                    ].add(attr["name"])
+                logging.info(
+                    f"Extracted the following assets to attributes: `{self.debug_asset_to_attributes}."
+                )
+
+    def validate_asset(self, asset):
+        if (
+            self.debug_metadata_path is None
+            or asset.resource_code not in self.debug_asset_to_attributes
+        ):
+            return True
+
+        expected_fields = self.debug_asset_to_attributes[asset.resource_code]
+        found_fields = set(asset.fields.keys())
+
+        missing_fields = expected_fields - found_fields
+        if len(missing_fields) > 0:
+            raise RuntimeError(
+                f"Asset `{asset}`` is missing the following fields: `{missing_fields}`."
+            )
+
+        extraneous_fields = found_fields - expected_fields
+        if len(extraneous_fields) > 0:
+            raise RuntimeError(
+                f"Asset `{asset}` contain unknown fields: `{extraneous_fields}`."
+            )
 
     @staticmethod
     def from_env():
@@ -41,6 +92,7 @@ class IngestionSessionContext:
                 "RELATIONSHIPS_OUTPUT_LOCATION",
                 f"file://{tempfile.mkdtemp()}/",
             ),
+            debug_metadata_path=os.getenv("debug_metadata_path", None),
         )
 
 
